# -*- coding: utf-8 -*-
"""
Calibration script for 3-axis accelerometer ADXL326

Python 2
02/2018
Author : Solène Kojtych
solene.kojtych@polymtl.ca
"""
import serial
import time
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import MaxNLocator
import os  
     
# %%------------------- Parameters -------------------------------------------

# Name of USB port used for Arduino
serialPortName = '/dev/ttyACM0'
# Path of the result folder
resultFolderPath="/home"
# test identification number
IDtest=1

# Accelerometer's parameters
accelerometerName="ADXL326"
axisNb = 3     

# Connexion parameters
dataRate = 57600  # data rate in bits per second (same as Arduino script)

# %%  ----------------- Initialization ------------------------------------

# Connexion via USB port
    # opens connexion
serialPort = serial.Serial(serialPortName, dataRate)    

    # erase remaining data in buffer
if serialPort.in_waiting>0:
    serialPort.reset_input_buffer
while serialPort.in_waiting>0:
    pass    

    # reset script on Arduino
serialPort.setDTR(False)     
time.sleep(0.022)          
serialPort.setDTR(True)    

# %%------------------- Acquisition --------------------------------------------
time.sleep(1)
continueAcquisition = True

# Calibration 
while continueAcquisition  is True :
    time.sleep(1)
    if serialPort.in_waiting>0:
           line = serialPort.readline()
           textLine = line.rstrip()
           print(line) 
           if textLine =='DEBUT':   # write 'BEGIN'
               continueAcquisition=False
    else:
        response = str(input('>> '))
        serialPort.write(response)   

# Reading of calibration results from serial port
calibrationValues = np.zeros([axisNb,3]) 
for axis in range(axisNb):
    line = serialPort.readline()
    textLine = line.rstrip()
    print("Axe "+str(axis)+" : "+textLine)
    # get values for -1, 0 and 1 g
    calibrationValues[axis,:] = textLine.split(';') 
        
# Connexion closure
serialPort.flushInput()        # erase buffer

print('Donnes restantes')      #'Remaining data'
print(serialPort.in_waiting)

serialPort.close()
print('Connexion fermee')     #'End of connexion'


# %%--------------------- Calculations and graphs -------------------------------------------

#----- Graphs
accelerationAbscissa = [-1,0,1]
graphicsTitles = ["axe X","axe Y","axe Z"]

for axis in range(axisNb):
    ax=plt.figure().gca()
    ax.yaxis.set_major_locator(MaxNLocator(integer=True))
    plt.xticks(np.arange(-1,2))    
    plt.plot(accelerationAbscissa,calibrationValues[axis,:])
    plt.scatter(accelerationAbscissa,calibrationValues[axis,:],
                facecolors='none', edgecolors='r')
    plt.ylabel("Tension relevee de 0 a 1023") # 'voltage between 0 and 1023
    plt.xlabel("Acceleration subie (g)")  #'acceleration (g)
    plt.title(graphicsTitles[axis])
    plt.show()

#---- Sensibility and bias calculation (values between 0 and 1023)
    
# Array with one line per axis (sensibility- bias)
axisResults =  np.zeros([axisNb,2])
for axis in range(axisNb):
    # linear regression (sensitivity = slope, biais = y-intercept)   
    axisResults[axis] = np.polyfit(accelerationAbscissa, 
               calibrationValues[axis], 1)   
    
# %% ----------------- Results printing ----------------------------
completeResultFolderPath=resultFolderPath+'/'+"resultats_calibration"

# Creation of a results folder if it doesn't exist
if not os.path.exists(completeResultFolderPath):
    os.makedirs(completeResultFolderPath)

# Results file creation
resultsFilePath = completeResultFolderPath+'/calibration_'
resultsFilePath=resultsFilePath+accelerometerName+'_'+str(IDtest)+'.txt'  
resultsFile = open(resultsFilePath, "w+")   
ordinateAxis=' '
for axis in range(axisNb):
    ordinateAxis =ordinateAxis+'   '+graphicsTitles[axis]
resultsFile.write('# Calibration '+accelerometerName+' run '+str(IDtest)+'\n')
resultsFile.write('# ---- MESURES ----\n')    # 'Measures
resultsFile.write("Axe         -1g           0g         1g \n")
for axis in range(axisNb):
    resultsFile.write("{}        {:.0f}         {:.0f}       {:.0f} \n"
                      .format(graphicsTitles[axis],calibrationValues[axis,0],
                       calibrationValues[axis,1],calibrationValues[axis,2]))                    

# Result file closure
resultsFile.close()   
    
# %%------------------ Calibration summary ----------------------

print('----------  Récapitulatif  ------------------') # 'Summary'
    # 'Accelerometer reference
print("Référence accéléro : {0}".format(accelerometerName)) 
    # results file's name
print("Nom du fichier de calibration produit : {0}".format(resultsFilePath)) 
    # 'Axis order'
print("Axes dans l'ordre : {}".format(ordinateAxis))   
print("Valeurs relevées")   # 'Values'
    # Axis -1g 0g 1g'
print("Axe         -1g           0g         1g") 
for axis in range(axisNb):
    print("{}        {:.0f}         {:.0f}       {:.0f}"
          .format(graphicsTitles[axis],calibrationValues[axis,0],
                  calibrationValues[axis,1],calibrationValues[axis,2]))    
print('----------------------------------------------')
