# -*- coding: utf-8 -*-
"""
Data acquisition for accelerometer ADXL326
(and possibly various accelerometer)

Python 2
02/2018
Author : Solène Kojtych
solene.kojtych@polymtl.ca
"""

import serial
import time

import numpy as np
import matplotlib.pyplot as plt
import os

# %%------------------- Parameters -----------------

# Name of USB port used for Arduino
serialPortName = '/dev/ttyACM0'
# Path of the result folder
resultFolderPath = "/home"
# test identification number
IDtest=1
# number of figures after comma for results
nbOfFigures = 3   

# Accelerometer's parameters
accelerometerName="ADXL326"   
axisNb = 3     
sensitivity = [18.8,18.8,19] # (betweeen 0 and 1023) for axis X,Y,Z
bias = [510.7,507.9,540.9]  # (betweeen 0 and 1023) for axis X,Y,Z

# Acquisition parameter
beepsNb = 0              # number of beeps before the acquisition starts

# Connexion parameters
dataRate  = 2000000  # data rate in bits per second (same as Arduino script)

# %%  ----------------- Initialization ------------------------------------

# Creation of a folder for results if it doesn't exist
completeResultFolderPath = resultFolderPath+'/resultats_acquisition'
if not os.path.exists(completeResultFolderPath ):
    os.makedirs(completeResultFolderPath )
    
# Creation of a folder for results of this particuliar test
completeResultFolderPathAccelero =completeResultFolderPath+'/'
completeResultFolderPathAccelero+=accelerometerName+'_'+str(IDtest)
if not os.path.exists(completeResultFolderPathAccelero ):
    os.makedirs(completeResultFolderPathAccelero)

# Text file opening
    # automatic increment of testNumber
testNumber = 1

folderName=completeResultFolderPathAccelero+"/mesures_%s" % testNumber+'.txt'
while os.path.exists(folderName):
    testNumber += 1
    folderName=completeResultFolderPathAccelero+"/mesures_%s"% testNumber+'.txt'

resultsFilePath = completeResultFolderPathAccelero +'/mesures_'
resultsFilePath+=str(testNumber)+'.txt'
resultsFile = open(resultsFilePath , "w+")    

# Connexion initialization  
    # Opening
serialPort = serial.Serial(serialPortName, dataRate)    

    # erase remaining data in buffer
if serialPort.in_waiting>0:
    serialPort.reset_input_buffer
print('Donnes restantes')       #'Remaining data'
print(serialPort.in_waiting)

    # reset script on Arduino
serialPort.setDTR(False) 
time.sleep(0.022)    
serialPort.setDTR(True)  

# Reading of the word "DEBUT" ('BEGIN')
line = serialPort.readline()       
line = line.decode("utf-8")
print(line)
serialPort.reset_input_buffer   
time.sleep(1)

#%%  -------------------- Sampling-----------------------------------------
response = '0'
print('Appuyez sur 1 pour commencer puis entree') #'Push 1 and enter
while response !='1':
    response = str(input('>> '))
    
# Delay
if beepsNb >0:    
    # Short beep
    order ="aplay "+resultFolderPath+'/bip0.wav'
    for i in range(0,beepsNb):    
        os.system(order)
        time.sleep(0.9)    
    # Long beep => acquisition starts
    order ="aplay "+resultFolderPath+'/bip1.wav'
    os.system(order) 
 
serialPort.write(response)    # sends response to Arduino

# Read data on serial port uniil word "FIN" appear ('END')
continueReading = True
while continueReading  is True :
    line = serialPort.readline()
    textLine = line.rstrip()   
    if textLine == 'FIN':    #'END'
        continueReading=False
    else:
        print(textLine)
        resultsFile.write(line)   # writes data

# %%---------------------Closure of serial connexion --------------------------
# File closure
resultsFile.close()

# erase remaining data in buffer
serialPort.flushInput() 
print('Donnes restantes')   #'Remaining data'
print(serialPort.in_waiting)

# Serial port closure
serialPort.close()
print('Connexion fermee')   #'End of connexion'
# %%-------------------- Post-processing --------------------------------------
graphsTitles = ["axe X","axe Y","axe Z"] # 'Axis X, axis Y, axis Z'

# File reading (time + accelerations in numerical values betwwen 0 and 1023)
data=np.genfromtxt(resultsFilePath, delimiter=';')
data = data[1:,:]                      # removes first line (numerical error)

    
# Acceleration calculations (in g)
physicalQuantity = np.copy(data)
for axis in range(axisNb):
    physicalQuantity[:,axis+1] = (data[:,axis+1]-bias[axis])/sensitivity[axis]

# Writing of accelerations in a new file (in g)
writingFormat = '%.'+str(nbOfFigures)+'f'
postProcessingFileName = completeResultFolderPathAccelero+'/accelerations_'
postProcessingFileName+=str(testNumber)+'.txt'
headerCSV = ['Temps'] +graphsTitles[:axisNb]  # 'Time'
np.savetxt(postProcessingFileName, physicalQuantity, delimiter=";",
           header=' '.join(headerCSV))

# %%------------------- Graphs  ----------------------------
# Initialization
time= physicalQuantity[:,0]/1000000      # time in seconds
samplingsNb = time.size
accelerations=np.zeros([samplingsNb,3])

# Graphs
for axis in range(axisNb):
    # Accelerations
    accelerationsAxis = physicalQuantity[:,axis+1]
    accelerations[:,axis] = accelerationsAxis

    # Graph for this axis
    plt.figure()
    plt.plot(time, accelerationsAxis)
    plt.title(graphsTitles[axis])
    plt.ylabel('Acceleration (g)')  
    plt.xlabel('Temps (s)')

